#include <ansi_c.h>
#include <visa.h>
#include <cvirte.h>     
#include <userint.h>
#include "RSSMU_Fading_Example.h"
#include "rssmu.h"

/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        rssmu_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", error_buffer); \
            rssmu_error_query (io, &error, error_message);\
            sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetWaitCursor (0); \
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);\
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);\
            MessagePopup ("Error", error_buffer); \
            rssmu_close(io); \
            break; \
        } \
    }
    
/* Globals */

ViInt32 panelHandle;
ViSession io;
ViStatus    error;
ViChar      error_message[256];
ViChar      error_buffer[256];

//ViString    GSM_STD[] = {"G6TU3","G6TU50","G6HT100","GRA250","GET50","GET100","GTU3","GTU50","GHT100"};
/* Local Functions */
int initialize (void);
            
/*= Main Function ===========================================================*/   
int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    panelHandle = LoadPanel (0, "RSSMU_Fading_Example.uir", PANEL); 
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    return 0;
}


/*===========================================================================*/
/* Function: Exit                                                            */
/* Purpose:  This is a callback function of Exit button. It closes the       */
/*           session and exits the example.                                  */
/*===========================================================================*/
int CVICALLBACK exitApp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{

    if (panel == panelHandle)
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz SMU Vector Signal Generator\n"
                              "instrument driver provided in the package.\n");

    return;
}


/*===========================================================================*/
/* Function: Apply Settings                                                  */
/* Purpose:  This is a callback function of Apply button. The values are     */
/*           send to the instrument                                          */
/*===========================================================================*/
int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32     standard, dedicated, speed_unit;
    ViBoolean   state;
    ViInt32     comm_speed;
    ViReal64    rf_level, rf_freq;
    
    switch (event)
    {
        case EVENT_COMMIT:
             SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 1);
             SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 1);
             ProcessDrawEvents ();

             if (initialize() < VI_SUCCESS) break;
             
             SetWaitCursor(1);
             
             CHECKERR (rssmu_SetDMAcctoStandard (io, RSSMU_PATH_A, RSSMU_DM_DIGITAL_STANDARD_GSM));
             GetCtrlVal (panelHandle, PANEL_DM_STATE, &state); 
             CHECKERR (rssmu_SetDMState (io, RSSMU_PATH_A, state));
             GetCtrlVal (panelHandle, PANEL_GSM_STANDARD, &standard); 
             CHECKERR (rssmu_SetFSIMStandard (io, RSSMU_PATH_A, standard)); 
             
             GetCtrlIndex (panelHandle, PANEL_DEDICATED, &dedicated);
             GetCtrlIndex (panelHandle, PANEL_SPEED_UNIT, &speed_unit);
             GetCtrlVal (panelHandle, PANEL_COMMON_SPEE, &comm_speed); 
             GetCtrlVal (panelHandle, PANEL_FADING_STATE, &state);
             CHECKERR (rssmu_SetFSIMSignaldedicatedTo (io, RSSMU_PATH_A, dedicated));
             CHECKERR (rssmu_SetFSIMSpeedUnit (io, RSSMU_PATH_A, speed_unit));
             CHECKERR (rssmu_SetFSIMCommonSpeedPaths (io, RSSMU_PATH_A, comm_speed));
             CHECKERR (rssmu_SetFSIMState (io, RSSMU_PATH_A, state));
             
             GetCtrlVal (panelHandle, PANEL_RF_LEVEL, &rf_level);
             GetCtrlVal (panelHandle, PANEL_RF_FREQ, &rf_freq); 
             GetCtrlVal (panelHandle, PANEL_RF_STATE, &state);
             CHECKERR (rssmu_SetRFAmplitude (io, RSSMU_PATH_A, rf_level));
             CHECKERR (rssmu_SetRFFrequency (io, RSSMU_PATH_A, rf_freq));
             CHECKERR (rssmu_SetOutputState (io, RSSMU_PATH_A, state));
             rssmu_close(io);
             
             SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
             SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);
             SetWaitCursor(0);

            break;
    }
    return 0;
}

/*===========================================================================*/ 
/* Local Functions Definitions                                               */
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset                                                       */
/*===========================================================================*/
int initialize (void)
{
    ViChar      instr_desc[256];
    ViBoolean   id;
    ViBoolean   rst;
    

    SetWaitCursor(0);
    GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
    GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
    GetCtrlVal (panelHandle, PANEL_RESET, &rst);
    SetWaitCursor(1);
    if ((error = rssmu_init (instr_desc, id, rst, &io)) < 0 )
    {
        rssmu_error_message(VI_NULL, error, error_message);
        sprintf (error_buffer, "Initialization Error:\n\n%s\n\nCheck your "
                "connections and make sure you have the right instrument descriptor.",
                 error_message);
        MessagePopup ("ERROR!", error_buffer);
        SetWaitCursor(0);
        return(-1);   
    }

    viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000);
    SetWaitCursor(0);
    return 0;
}
